#include <iostream>
#include <math.h>
#include "gang.h"

using namespace std;


Gang::Gang(QSlider* s_, QLineEdit* le_, 
		const float minv_, const float maxv_, 
		const float vv, const bool logs) : 
			s(s_), le(le_), minv(minv_), maxv(maxv_), defaultv(vv), logscaling(logs)
{
	//cout << "Gang::Gang()" << endl;
	connect( s,  SIGNAL( valueChanged(int) ), this, SLOT(sliderMoved(int) ) );
	connect( le, SIGNAL( returnPressed() ),   this, SLOT(textChanged()    ) );
	
	/*
	value_from_text = false;
	v_ = defaultv;
	s->setValue( v2p(v()) );
	changed_ = false;
	*/
	setDefault();
	return;
}

float Gang::p2v(const int p) const
{
	float x = (p-s->minValue())/( (float) (s->maxValue() - s->minValue() ) ) ;
	if( logscaling ) {
		//cout << "p:  " << p << ", x:  " << x << ", " << minv*exp(log(maxv/minv)*x ) << endl;
		return minv*exp(log(maxv/minv)*x );
	}
	return (maxv-minv)*x + minv; 
}

int Gang::v2p(const float x) const
{
	float y = (x - minv)/(maxv - minv);
	if( logscaling ) {
		y = (log(x)-log(minv))/(log(maxv)-log(minv));
		//cout << "x:  " << x << ", y:  " << y << ", " << log(x) << endl;
	}
	return  (int) ( (s->maxValue() - s->minValue() )*y + s->minValue() );
}

void Gang::sliderMoved(int p) 
{
	if( value_from_text ) {
		value_from_text = false;
		return;
	}
	v_ = p2v(p);
	le->setText( QString("%1").arg( v() )  );
	changed_ = true;
	return;
}

void Gang::textChanged()
{
	bool ok;
	
	float x = le->text().toFloat(&ok);
	if( ! ok ) 
		x = v();
	v_ = x;
	value_from_text = true;
	s->setValue( v2p(v() ) );
	changed_ = true;
	return;
}

void Gang::setDefault()
{
	v_ = defaultv;
	le->setText( QString("%1").arg(v()));
	value_from_text = true;
	s->setValue( v2p(v()) );
	changed_ = false;
	return;
}

QString Gang::flag(const QString f) const
{
    if( ! changed() )
	return "";
    return QString(" %1 %2").arg(f).arg(v());
}
QString Gang::fname(const QString f) const
{
    if( ! changed() )
	return "";
    return QString(".%1%2").arg(f).arg(v());
}
