/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include <iostream>
#include <qfiledialog.h>
#include <qapplication.h>
//#include <fstream>
#include "imageviewer.h"

using namespace std;

extern QString cmdlinefname;

bool getPFSsize(const QByteArray &buffer, int &height, int &width)
{
    height = width = 0;
    // Check this is a PFS stream
    bool pfscheck = (buffer[0] == 'P') && (buffer[1] == 'F') && (buffer[2] == 'S' && (buffer[3] == '1'));
    if( ! pfscheck ) 
	return false;
    // Without too much errorchecking, I am going to get the width and height
    QString str;
    for(uint i  = 5; i < buffer.size(); i++) {
	str.append(buffer[i]);
	if( buffer[i] == ' ' ) {
	    width = str.toInt();
	    str = "";
	}
	if( buffer[i] == '\n' )
	    break;
    }   
    height = str.toInt();
    return true;
}

//=============================================================================
void TMDialog::init()
{
//        name,                           Slider,       Line Edit,     Range:  Min,   Max,   Default, true=log scaling 

	// ashikhmin02
	contrastGang = 		new Gang(contrastSlider, contrastLineEdit, 	0, 	1, 	0.5);

	// drago03
	biasGang = 		new Gang(biasSlider, biasLineEdit, 		0.5, 	1, 	0.85);

	// durand02
	spatialGang = 		new Gang(spatialSlider, spatialLineEdit, 	0, 	10, 	8);
	rangeGang = 		new Gang(rangeSlider, rangeLineEdit, 		0, 	10, 	0.4);
	baseGang = 		new Gang(baseSlider, baseLineEdit, 		0, 	10, 	5.0);

	// fattal02
	alphaGang = 		new Gang(alphaSlider, alphaLineEdit, 		1e-3, 	1, 	1e-1,	true);
	betaGang = 		new Gang(betaSlider, betaLineEdit, 		0.5, 	1, 	.8);
	saturation2Gang = 		new Gang(saturation2Slider, saturation2LineEdit, 0, 	1, 	0.6);

	// pattanaik00
	multiplierGang = 	new Gang(multiplierSlider, multiplierLineEdit, 	1e1, 	1e4, 	1, 	true);
	coneGang = 		new Gang(coneSlider, coneLineEdit, 		0, 	1, 	0.5);
	rodGang = 		new Gang(rodSlider, rodLineEdit, 		0, 	1, 	0.5);

	// reinhard02
	keyGang = 		new Gang(keySlider, keyLineEdit, 		0, 	1, 	0.18);
	phiGang = 		new Gang(phiSlider, phiLineEdit, 		0, 	50, 	1);
	range2Gang = 		new Gang(range2Slider, range2LineEdit, 	2, 	15, 	8);
	lowerGang = 		new Gang(lowerSlider, lowerLineEdit, 		1, 	100, 	1);
	upperGang = 		new Gang(upperSlider, upperLineEdit, 		1, 	100, 	43);

	// reinhard04
	brightnessGang = 	new Gang(brightnessSlider, brightnessLineEdit,	-1, 	1, 	0);
	saturationGang = 	new Gang(saturationSlider, saturationLineEdit, 	0, 	1, 	0.1);

	loadproc = new QProcess(this);
	connect( loadproc, SIGNAL( processExited() ), this, SLOT( inputReady() ) );
 	
	resizeproc = new QProcess(this);
	connect( resizeproc, SIGNAL( processExited() ), this, SLOT( resizeReady() ) );
	connect( resizeproc, SIGNAL( wroteToStdin()  ), resizeproc, SLOT( closeStdin() ) );
 	
	if( cmdlinefname == "") {
		selectInput();
	}
	else {
		inputFile = cmdlinefname;
		inputLineEdit->setText(inputFile);
		loadInput();
	}
	currImgDialog = 0;
	
	helpDialog = new HelpDialog(this);
	return;
}

void TMDialog::getToneMapCmd( QString &cmd, QString &args, QString &fname )
{

    cmd = ""; //"pfstmo_";
    args = "";
    switch( filterTab->currentPageIndex() ) {
    case 0:
	cmd = "ashikhmin02";
	if( simpleCheckBox->isOn() )
	    args += " -s";
	args += contrastGang->flag("--local-contrast");
	args += eq2RadioButton->isOn() ? " --equation 2" : " --equation 4";
	if( simpleCheckBox->isOn() )
	    fname += ".s";
	fname += "." + cmd;
	fname += contrastGang->fname("l");
	fname += eq2RadioButton->isOn() ? ".eq2" : ".eq4";
	break;
    case 1:
	cmd = "drago03";
	args += biasGang->flag("--bias");
	fname += "." + cmd;
	fname += biasGang->fname("b");
	break;
    case 2:
	cmd = "durand02";
	args += spatialGang->flag("--sigma-s");
	args += rangeGang->flag("--sigma-r");
	args += baseGang->flag("--base-contrast");
	fname += "." + cmd;
	fname += spatialGang->fname("ss");
	fname += rangeGang->fname("sr");
	fname += baseGang->fname("b");
	break;
    case 3:
	cmd = "fattal02";
	args += alphaGang->flag("--alfa");
	args += betaGang->flag("--beta");
	args += saturation2Gang->flag("--saturation");
	fname += "." + cmd;
	fname += alphaGang->fname("a");
	fname += betaGang->fname("b");
	fname += saturation2Gang->fname("s");
	break;
    case 4:
	cmd = "pattanaik00";
	args += multiplierGang->flag("--mul");
	if( localCheckBox->isOn() )
	    args += " --local";
	args += coneGang->flag("--cone");
	args += rodGang->flag("--rod");
	fname += "." + cmd;
	fname += multiplierGang->fname("m");
	if( localCheckBox->isOn() )
	    fname += ".l";
	fname += coneGang->fname("c");
	fname += rodGang->fname("r");
	break;
    case 5:
	cmd = "reinhard02";
	args += keyGang->flag("--key");
	if( scalesCheckBox->isOn() )
	    args += " --scales";
	args += phiGang->flag("--phi");
	args += range2Gang->flag("--range");
	args += lowerGang->flag("--lower");
	args += upperGang->flag("--upper");
	fname += "." + cmd;
	fname += keyGang->fname("k");
	if( scalesCheckBox->isOn() )
	    fname += ".s";
	fname += phiGang->fname("p");
	fname += range2Gang->fname("r");
	fname += lowerGang->fname("l");
	fname += upperGang->fname("u");	
	break;
    case 6:
	cmd = "reinhard04";
	fname += "." + cmd;
	args += brightnessGang->flag("--brightness");
	args += saturationGang->flag("--saturation");
	fname += "." + cmd;
	fname += brightnessGang->fname("b");
	fname += saturationGang->fname("s");
	break;
    };
     return;
}

//============================================================================
void TMDialog::runToneMap()
{
	if(  workbuffer.size() == 0 ) {
		QMessageBox::warning(this,"","Please select a <b>Work Size</b>",
				QMessageBox::Ok, QMessageBox::NoButton);
		return;
	}
	QString tm_cmd;
	QString tm_args;
	
	QFileInfo finfo(inputFile);
	QString tm_fname = finfo.dirPath() + "/" + finfo.baseName();
	
	getToneMapCmd(tm_cmd, tm_args, tm_fname);
	
	if( newWinCheckBox->isChecked() ) {
		currImgDialog = 0;
	}
	if( currImgDialog == 0 ) {
		currImgDialog = new ImageDialog(this);
	}
	currImgDialog->show();
	currImgDialog->set(workbuffer,"pfstmo_"+tm_cmd,tm_args,tm_cmd + " " + tm_args,tm_fname + ".png");
	
	return;	
}

//============================================================================
void TMDialog::ashikhminReset()
{
	contrastGang->setDefault();
}


//============================================================================
void TMDialog::dragoReset()
{
	biasGang->setDefault();
}


//============================================================================
void TMDialog::durandReset()
{
	spatialGang->setDefault();
	rangeGang->setDefault();
	baseGang->setDefault();
}


//============================================================================
void TMDialog::fattalReset()
{
	alphaGang->setDefault();
	betaGang->setDefault();
	saturation2Gang->setDefault();
}


//============================================================================
void TMDialog::pattanaikReset()
{
	multiplierGang->setDefault();
	coneGang->setDefault();
	rodGang->setDefault();
}


//============================================================================
void TMDialog::reinhard02Reset()
{
	keyGang->setDefault();
	phiGang->setDefault();
	range2Gang->setDefault();
	lowerGang->setDefault();
	upperGang->setDefault();
}


//============================================================================
void TMDialog::reinhart04Reset()
{
	brightnessGang->setDefault();
	saturationGang->setDefault();
}


//============================================================================
void TMDialog::gammaReset()
{
	gammaGang->setDefault();
}

//**************************************************************************************************
//============================================================================
void TMDialog::selectInput()
{
	QString cwd = ".";
	QString filetypes;
	filetypes += "OpenEXR (*.exr);;";
	filetypes += "Radiance RGBE (*.pic,*.hdr);;";
	filetypes += "TIFF (*.tiff,*.tif);;";
	filetypes += "PNM, PPM  (*.ppm,*.pnm);;";
	filetypes += "JPEG (*.jpeg,*.jpg);;";
	filetypes += "PNG (*.png);;";
	filetypes += "PFS (*.pfs);;";
	filetypes += "PFM (*.pfm);;";
	filetypes += "DPX (*.dpx);;";
	filetypes += "GIF (*.gif);;";
	filetypes += "BMP (*.bmp);;";
	filetypes += "EPS (*.eps);;";
	filetypes += "All Files (*)";
	inputFile = QFileDialog::getOpenFileName(inputFile, filetypes,this,
					"Choose", "Select an Image");
	if( inputFile.isEmpty() ) {
		return;
	}
	inputLineEdit->setText(inputFile);
	loadInput();
	return;
}


//============================================================================
void TMDialog::loadInput()
{
	origbuffer.resize(0);
	workbuffer.resize(0);
	loadproc->clearArguments();
	loadproc->addArgument("pfsin");
	loadproc->addArgument(inputFile);
	if ( ! loadproc->start() ) {
		cout << "TMDialog::loadInput:Failure to start" << endl;
	}
	return;
}


//============================================================================
void TMDialog::inputReady()
{
	origbuffer = loadproc->readStdout();
	if( origbuffer.size() == 0 ) {
		QString errstring = loadproc->readLineStderr();
		QMessageBox::critical(this,"pfsin Error",errstring,QMessageBox::Ok,QMessageBox::NoButton);
		selectInput();
	}
	origbuffer.detach();
    
	int height = 0;
	int width = 0;
	if( ! getPFSsize(origbuffer,height,width) ) {
		QMessageBox::critical(this,
				"pfs Error","Loaded image doesn't look like a PFS stream",
				QMessageBox::Ok,QMessageBox::NoButton);
		selectInput();
    	}

	// Lets make a list of possible sizes
	// This is just a guess at a list
	sizes.resize(0);
	for(int x = 128; x <= width; x *= 2) {
		if( x >= width )
		break;
		sizes.push_back(x);
		if( 3*(x/2) >= width )
		break;
		sizes.push_back(3*(x/2));
	}
	sizes.push_back(width);
	sizeComboBox->clear();
	float r = ( (float)height )/( (float) width);
	for(uint i = 0; i < sizes.size(); i++) {
		sizeComboBox->insertItem( QString("%1x%2").arg(sizes[i]).arg( (int)(r*sizes[i]) ), i );
	}
	return;
}


//============================================================================
void TMDialog::setWorkSize( int index)
{
	buttonApply->setEnabled(false);
	workbuffer.resize(0);
	resizeproc->clearArguments();
	resizeproc->addArgument("pfssize");
	resizeproc->addArgument( QString("-x %1").arg(sizes[index]) );
	if ( ! resizeproc->start() ) {
		cout << "TMDialog::setWorkSize:Failure to start" << endl;
		buttonApply->setEnabled(true);
	}
	resizeproc->writeToStdin( origbuffer );

	return;
}


//============================================================================
void TMDialog::resizeReady()
{
	workbuffer = resizeproc->readStdout();
	buttonApply->setEnabled(true);
	
	if( currImgDialog == 0 ) {
		currImgDialog = new ImageDialog(this);
	}
	currImgDialog->show();
	currImgDialog->set(workbuffer,"","",inputFile,inputFile + ".png");
	
	return;
}

extern QApplication *qApp;

//============================================================================
void TMDialog::quiting()
{
	qApp->quit();
	reject();
}


void TMDialog::showHelp()
{
    helpDialog->show();
}
